function [varargout] = renflow(params)

plotevery = 0;

% Set some default parameters, if none are provided or if params is a string.
if nargin < 1 || strcmp(params,'case1') || strcmp(params,'type1')
  clear params
  params.type = 1;
  params.Nstep = 2000; params.Nreal = 100; params.N = 512;
  params.kappa = 1e-3; params.alpha = pi; params.p = 1;
elseif strcmp(params,'case2') || strcmp(params,'type2')
  clear params
  params.type = 2;
  params.Nstep = 2000; params.Nreal = 100; params.N = 512;
  params.kappa = 1e-3; params.alpha = pi; params.p = 1;
elseif strcmp(params,'gammap_case1') || strcmp(params,'gammap_case1')
  clear params
  params.type = 1;
  params.Nstep = 2000; params.Nreal = 100; params.N = 2048;
  params.kappa = 1e-3; params.alpha = pi; params.p = -2:5;
elseif strcmp(params,'gammap_case2') || strcmp(params,'gammap_type2')
  clear params
  params.type = 2;
  params.Nstep = 2000; params.Nreal = 3000; params.N = 2048;
  params.kappa = 1e-4; params.alpha = pi; params.p = -1:.5:5;
elseif strcmp(params,'anim1')
  % 'animate' the type1 case
  clear params
  plotevery = 1;
  params.type = 1;
  params.Nstep = 2000; params.Nreal = 1; params.N = 512;
  params.kappa = 1e-3; params.alpha = pi; params.p = 1;
elseif strcmp(params,'anim2')
  % 'animate' the type2 case
  clear params
  plotevery = 1;
  params.type = 2;
  params.Nstep = 2000; params.Nreal = 1; params.N = 512;
  params.kappa = 1e-3; params.alpha = pi; params.p = 1;
end
Nstep = params.Nstep; Nreal = params.Nreal; N = params.N;
alpha = params.alpha; p = params.p;

kmin = floor(-(N-1)/2); kmax = floor((N-1)/2); k = [0:kmax kmin:-1];
x = linspace(0,2*pi,N+1); x = x(1:end-1);

switch params.type
 case 1
  % randn(Nreal,1)*sin(x) = kron(sin(x),randn(Nreal,1))
  advec = @() exp(-i*alpha*randn(Nreal,1)*sin(x));
 case 2
  advec = @() exp(-i*alpha*sin(tensorsum(x,2*pi*rand(Nreal,1))));
end

rng('default')
Cnorm = zeros(Nreal,Nstep+1,length(p)); C = ones(Nreal,N);
Cnorm(:,1,:) = Cnorms(C,p);
diff = diag(sparse(exp(-params.kappa*k.^2)));
for n = 1:Nstep
  if ~mod(n,10), fprintf('step %d\n',n); end
  C = advec().*C;                         % advection step
  Ck = fft(C,[],2);                       % Fourier transform
  Ck = Ck*diff;                           % diffusion step
  C = ifft(Ck,[],2);                      % inverse Fourier transform
  Cnorm(:,n+1,:) = Cnorms(C,p);
  if ~mod(n,plotevery)
    if n == plotevery && ishandle(1), close(1); end
    figure(1)
    subplot(2,1,1)
    semilogy(0:n,Cnorm(1,1:n+1),'r-'), hold on
    xlabel('$n$','Interpreter','LaTeX')
    ylabel('$\|\hat C\|$','Interpreter','LaTeX')
    subplot(2,1,2)
    Cplot = C(1,:)/Cnorm(1,n+1,1);
    plot(x,abs(Cplot)), hold on
    plot(x,real(Cplot),'r--')
    plot(x,imag(Cplot),'g--')
    axis([min(x) max(x) -1.1*max(abs(Cplot)) 1.1*max(abs(Cplot))])
    xlabel('$x$','Interpreter','LaTeX')
    ylabel('$\hat C/\|C\|$','Interpreter','LaTeX')
    hold off
    drawnow %, pause(.01)
  end
end

if plotevery, figure(1), subplot(2,1,1), hold off, end

if nargout > 0
  varargout{1} = squeeze(mean(Cnorm)); % average over realizations
  if nargout > 1
    varargout{2} = params;
  end
elseif length(p) == 1 && ~plotevery
  % plot the final concentration for the last realization
  if ishandle(1), close(1); end
  figure(1)
  plot(x,abs(C(end,:))/Cnorm(end,end),'LineWidth',2), hold on
  plot(x,real(C(end,:))/Cnorm(end,end),'r--','LineWidth',2)
  plot(x,imag(C(end,:))/Cnorm(end,end),'g--','LineWidth',2)
  xlabel('$n$','Interpreter','LaTeX','FontSize',22)
  ylabel('$\|\hat C\|$','Interpreter','LaTeX','FontSize',22)
  axis tight
  xlabel('$x$','Interpreter','LaTeX','FontSize',22)
  ylabel('$\hat C/\|C\|$','Interpreter','LaTeX','FontSize',22)
  set(gca,'FontSize',18,'FontName','Times')
  hold off
  print -dpdf renflow_C.pdf

  if ishandle(2), close(2); end
  figure(2)
  for r = 1:min(Nreal,10)
    % plot the norm at each step for each realization
    semilogy(0:Nstep,Cnorm(r,:),'r-','LineWidth',2), hold on
    xlabel('$n$','Interpreter','LaTeX','FontSize',22)
    ylabel('$\|\hat C\|$','Interpreter','LaTeX','FontSize',22)
    set(gca,'FontSize',18,'FontName','Times')
  end
  hold off
  print -dpdf renflow_Cnorm.pdf
end

%=====================================================================
function Cp = Cnorms(C,p)

Cp = zeros(size(C,1),length(p));
for ip = 1:length(p)
  Cp(:,ip) = sqrt(sum(C.*conj(C),2)/size(C,2)).^p(ip);
end
